"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _orderBy2 = _interopRequireDefault(require("lodash/orderBy"));

var _reselect = require("reselect");

var getActivities = function getActivities(state) {
  return state.conversation.get('activities');
};

var getParticipants = function getParticipants(state) {
  return state.conversation.get('participants');
};

var getUsers = function getUsers(state) {
  return state.users;
};

var getTypingIndicators = function getTypingIndicators(state) {
  return state.indicators.get('typing');
};

var getSpark = function getSpark(state, ownProps) {
  return ownProps.sparkInstance || state.spark.get('spark');
};

var READ_RECEIPTS_SHOWN_LIMIT = 10;
var getCurrentUser = (0, _reselect.createSelector)([getUsers], function (users) {
  return users.getIn(['byId', users.get('currentUserId')]);
});
var getReadReceipts = (0, _reselect.createSelector)([getCurrentUser, getActivities, getParticipants, getTypingIndicators], function (currentUser, activities, participants, typing) {
  var activity = activities.last();
  var readParticipants = participants.filter(function (participant) {
    return activity && currentUser && participant.get('id') !== currentUser.id && participant.getIn(['roomProperties', 'lastSeenActivityUUID']) === activity.id;
  }).toJS();
  var mappedParticipants = readParticipants.map(function (participant) {
    var participantId = participant.id; // Typing events don't give us user IDs, only emails.

    var isTyping = typing.has(participant.emailAddress);
    return {
      displayName: participant.displayName,
      isTyping: isTyping,
      userId: participantId
    };
  });
  var sortedParticipants = (0, _orderBy2.default)(mappedParticipants, 'isTyping', 'desc');
  var visibleUsers = sortedParticipants.slice(0, READ_RECEIPTS_SHOWN_LIMIT);
  var hiddenUsers = sortedParticipants.slice(READ_RECEIPTS_SHOWN_LIMIT);
  return {
    hiddenUsers: hiddenUsers,
    visibleUsers: visibleUsers
  };
});
var getReadReceiptsProps = (0, _reselect.createSelector)([getReadReceipts, getSpark], function (readReceipts, sparkInstance) {
  return {
    readReceipts: readReceipts,
    sparkInstance: sparkInstance
  };
});
var _default = getReadReceiptsProps;
exports.default = _default;